@@ -1,11 +1,11 @@
 /* 
  * This file part of StarDict - A international dictionary for GNOME.
  * http://stardict.sourceforge.net
- * Copyright (C) 2005 Evgeniy <dushistov@mail.ru>
+ * Copyright (C) 2005-2006 Evgeniy <dushistov@mail.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,60 +25,17 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <cstdlib>
+#include <gtk/gtk.h>
 
-#if !defined(_WIN32) && !defined(WITHOUT_GNOME)
+#ifdef CONFIG_GNOME
 #  include <libgnome/libgnome.h>
 #  include <libgnomeui/libgnomeui.h>
 #elif defined(_WIN32)
 #  include <gdk/gdkwin32.h>
 #endif
 
-#include "conf.h"
-#include "stardict.h"
-
 #include "utils.h"
 
-#ifdef WITHOUT_GNOME
-static void spawn_command(const gchar *exe, const gchar *arg)
-{
-  gchar *cmd=g_strdup_printf("%s '%s'", exe, arg);
-  g_spawn_command_line_async(cmd, NULL);
-  g_free(cmd);
-}
-#endif
-
-void play_wav_file(const std::string& filename)
-{
-#ifdef _WIN32
-  PlaySound(filename.c_str(), 0, SND_ASYNC | SND_FILENAME);
-#elif !defined(WITHOUT_GNOME)
-  gnome_sound_play(filename.c_str());
-#else
-	const std::string &playcmd=
-		conf->get_string("/apps/stardict/preferences/dictionary/play_command");
-  spawn_command(playcmd.c_str(), filename.c_str());
-#endif
-}
-
-void show_help(const gchar *section)
-{
-#ifdef _WIN32
-  gchar *filename = g_strdup_printf(_("%s\\help\\stardict.chm"), gStarDictDataDir.c_str());
-  ShellExecute((HWND)(GDK_WINDOW_HWND(gpAppFrame->window->window)), "OPEN", filename, NULL, NULL, SW_SHOWNORMAL);
-  g_free(filename);
-#elif !defined(WITHOUT_GNOME)
-  gnome_help_display ("stardict.xml", section, NULL);
-#endif
-}
-
-void show_url(const gchar *url)
-{
-#ifdef _WIN32
-  ShellExecute((HWND)(GDK_WINDOW_HWND(gpAppFrame->window->window)), "OPEN", url, NULL, NULL, SW_SHOWNORMAL);
-#elif !defined(WITHOUT_GNOME)
-  gnome_url_show(url, NULL);
-#endif
-}
 
 void ProcessGtkEvent()
 {
@@ -86,12 +43,15 @@ void ProcessGtkEvent()
     gtk_main_iteration();
 }
 
-//sinse glib 2.6 we have g_get_user_config_dir
-//but because of compability with other 2.x...
 std::string get_user_config_dir()
 {
+        const gchar *config_path_from_env = g_getenv("STARDICT_CONFIG_PATH");
+        if (config_path_from_env)
+            return config_path_from_env;
 #ifdef _WIN32
-	return gStarDictDataDir;
+	std::string res = g_get_user_config_dir();
+	res += G_DIR_SEPARATOR_S "StarDict";
+	return res;
 #else
 	std::string res;
 	gchar *tmp = g_build_filename(g_get_home_dir(), ".stardict", NULL);
@@ -179,8 +139,65 @@ GdkPixbuf *load_image_from_file(const std::string& filename)
 	return res;
 }
 
-void play_sound_on_event(const gchar *eventname)
+static gchar * byte_to_hex(unsigned char nr) {
+	gchar *result = NULL;
+
+	result = g_strdup_printf("%%%x%x", nr / 0x10, nr % 0x10);
+	return result;
+}
+
+char *common_encode_uri_string(const char *string)
 {
-	if (conf->get_bool("/apps/stardict/preferences/dictionary/enable_sound_event"))
-	  play_wav_file(gStarDictDataDir+ G_DIR_SEPARATOR_S "sounds" G_DIR_SEPARATOR_S +eventname+".wav");
+	gchar		*newURIString;
+	gchar		*hex, *tmp = NULL;
+	int		i, j, len, bytes;
+
+	/* the UTF-8 string is casted to ASCII to treat
+	   the characters bytewise and convert non-ASCII
+	   compatible chars to URI hexcodes */
+	newURIString = g_strdup("");
+	len = strlen(string);
+	for(i = 0; i < len; i++) {
+		if(g_ascii_isalnum(string[i]) || strchr("-_.!~*'()", (int)string[i]))
+		   	tmp = g_strdup_printf("%s%c", newURIString, string[i]);
+		else if(string[i] == ' ')
+			tmp = g_strdup_printf("%s%%20", newURIString);
+		else if((unsigned char)string[i] <= 127) {
+			tmp = g_strdup_printf("%s%s", newURIString, hex = byte_to_hex(string[i]));g_free(hex);
+		} else {
+			bytes = 0;
+			if(((unsigned char)string[i] >= 192) && ((unsigned char)string[i] <= 223))
+				bytes = 2;
+			else if(((unsigned char)string[i] > 223) && ((unsigned char)string[i] <= 239))
+				bytes = 3;
+			else if(((unsigned char)string[i] > 239) && ((unsigned char)string[i] <= 247))
+				bytes = 4;
+			else if(((unsigned char)string[i] > 247) && ((unsigned char)string[i] <= 251))
+				bytes = 5;
+			else if(((unsigned char)string[i] > 247) && ((unsigned char)string[i] <= 251))
+				bytes = 6;
+				
+			if(0 != bytes) {
+				if((i + (bytes - 1)) > len) {
+					g_warning(("Unexpected end of character sequence or corrupt UTF-8 encoding! Some characters were dropped!"));
+					break;
+				}
+
+				for(j=0; j < (bytes - 1); j++) {
+					tmp = g_strdup_printf("%s%s", newURIString, hex = byte_to_hex((unsigned char)string[i++]));
+					g_free(hex);
+					g_free(newURIString);
+					newURIString = tmp;
+				}
+				tmp = g_strdup_printf("%s%s", newURIString, hex = byte_to_hex((unsigned char)string[i]));
+				g_free(hex);
+			} else {
+				/* sh..! */
+				g_error("Internal error while converting UTF-8 chars to HTTP URI!");
+			}
+		}
+		g_free(newURIString); 
+		newURIString = tmp;
+	}
+	return newURIString;
 }
